package org.amse.vbut.vzab;

/**
 * Board rectangle class
 * 
 * @author But Vasiliy
 *
 */
public class BoardRectangle {

	private final int myX0;
	private final int myY0;
	
	private final int myX1;
	private final int myY1;
	
	/**
	 * Constructs BoardRectangle object with specified coordinates
	 * of top-left (x0, y0) and bottom-right (x1, y1) corners. 
	 */
	public BoardRectangle(int x0, int y0, int x1, int y1) {
		myX0 = x0;
		myY0 = y0;
		myX1 = x1;
		myY1 = y1;
	}

	/**
	 * Returns x-coordinate of the top-left corner.
	 * @return x-coordinate of the top-left corner.
	 */
	public int getX0() {
		return myX0;
	}
	
	/**
	 * Returns y-coordinate of the top-left corner.
	 * @return y-coordinate of the top-left corner.
	 */
	public int getY0() {
		return myY0;
	}
	
	/**
	 * Returns x-coordinate of the bottom-right corner.
	 * @return x-coordinate of the bottom-right corner.
	 */
	public int getX1() {
		return myX1;
	}
	
	/**
	 * Returns y-coordinate of the bottom-right corner.
	 * @return y-coordinate of the bottom-right corner.
	 */
	public int getY1() {
		return myY1;
	}

	/**
	 * Tests if the specified point is in this rectangle.
	 * @return <tt>true</tt> if the specified point is in 
	 * this rectangle; <tt>false</tt> otherwise.
	 */
	public boolean inRect(int x, int y) {
		return x >= myX0 && x <= myX1 && y >= myY0 && y <= myY1;
	}
	
	/**
	 * Tests if the specified point is in this rectangle.
	 * @return <tt>true</tt> if the specified point is in 
	 * this rectangle; <tt>false</tt> otherwise.
	 */
	public boolean inRect(BoardPoint bp) {
		return inRect(bp.getX(), bp.getY()); 
	}
}
