package org.amse.vbut.vzab.model.impl;

import org.amse.vbut.vzab.model.*;

class Board implements IBoard {

	private final IPiece myMap[][];

	public Board(int w, int h) {
		myMap = new Piece[w][h];
		clear();
	}

	public final void clear() {
		for (int y = 0; y < getHeight(); ++y) {
			for (int x = 0; x < getWidth(); ++x) {
				myMap[x][y] = null;
			}
		}
	}

	public IPiece getPiece(int x, int y) {
		if (x < 0 || x >= getWidth() || y < 0 || y >= getHeight()) {
			return null;
		}
		return myMap[x][y];
	}

	public int getHeight() {
		if (myMap.length == 0) {
			return 0;
		}
		return myMap[0].length;
	}

	public int getWidth() {
		return myMap.length;
	}

	public void setPiece(int x, int y, IPlayer player) {
		myMap[x][y] = new Piece(player);
	}

	public String toString() {
		StringBuffer sb = new StringBuffer("[width=" + getWidth() 
				+ ",height=" + getHeight());

		for (int y = 0; y < getHeight(); ++y) {
			for (int x = 0; x < getWidth(); ++x) {
				IPiece f = getPiece(x, y);
				if (f != null) {
					sb.append("\n[x=" + x + ",y=" + y + "," 
							+ f.toString() + "]");
				}
			}
		}
		
		sb.append("]");

		return sb.toString();
	}

	public void removePiece(int x, int y) {
		myMap[x][y] = null; 
	}
}
