package org.amse.vbut.vzab.model.impl;

import org.amse.vbut.vzab.model.*;

import javax.swing.*;

/**
 * Model factory
 * 
 * @author But Vasiliy
 *
 */
public class ModelFactory {

	/**
	 * Creates new IGame object with specified width, height and players.
	 * @return new IGame object with specified width, height and players.
	 */
	public static IGame newGame(int w, int h, java.util.List<IPlayer> players) {
		return new Game(w, h, players);
	}

	/**
	 * Creates new IPlayer with specified name, isComputer flag and
	 * imageAlive, imageKilled, imageBase, imageBaseKilled images.
	 * Sets path of new player to path property.
	 * @return new IPlayer object.
	 */
	public static IPlayer newPlayer(String name, boolean isComputer, 
			ImageIcon imageAlive, ImageIcon imageKilled, ImageIcon imageBase, 
			ImageIcon imageBaseKilled, String path) {
		
		return new Player(name, isComputer, imageAlive, imageKilled, imageBase, 
				imageBaseKilled, path);
	}
	
	/**
	 * Creates new IPlayer object with specified name, isComputer flag and path.
	 * @return new IPlayer object.
	 */
	public static IPlayer newPlayer(String name, boolean isComputer, String path) {
		return new Player(name, isComputer, null, null, null, null, path);
	}

	/**
	 * Creates new IPlayer object with specified name, isComputer flag.
	 * @return new IPlayer object.
	 */
	public static IPlayer newPlayer(String name, boolean isComputer) {
		return new Player(name, isComputer, null, null, null, null, "");
	}

	/**
	 * Creates new IPlayer object with specified name.
	 * @return new IPlayer object.
	 */
	public static IPlayer newPlayer(String name) {
		return new Player(name, false, null, null, null, null, "");
	}

	/**
	 * Creates new copy of the specified player.
	 * @return new copy of the specified player.
	 */
	public static IPlayer newPlayer(IPlayer player) {
		return new Player(player);
	}
	
	/**
	 * Creates new IPLayerShell with the specified parent player.
	 * @param player - parent player
	 * @param isnew - value of the isnew flag
	 * @return new IPLayerShell object.
	 */
	public static IPlayerShell newPlayerShell(IPlayer player, boolean isnew) {
		return new PlayerShell(player, isnew);
	}
}
