package org.amse.vbut.vzab.tests;

import junit.framework.TestCase;

import org.amse.vbut.vzab.io.*;
import org.amse.vbut.vzab.io.impl.*;
import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.view.*;
import org.amse.vbut.vzab.*;

import java.io.File;

public class IOTests extends TestCase {

	private static final int TEST_COUNT = 2; 
	
	public void testReader() {
		final IReader reader = IOFactory.newReader();

		int i = 0;
		
		try {

			for (i = 0; i < TEST_COUNT; ++i) {
				
				final String filename = "testfiles" + File.separator 
					+ "test" + i + "." + Main.GAME_EXTENSION;
				
				reader.readGame(filename,
						IGame.MIN_WIDTH, IGame.MIN_HEIGHT, 20, 20);
			}
			
		} catch (VZabException e) {
			fail("Failed test number = " + i + "\n" +
					"Error: " + e.getMessage());
		} catch (RuntimeException e) {
			fail("Failed test number = " + i + "\n" +
					"Uncknown exception");
			e.printStackTrace();
		}
	}
	
	public void testWriter() {
		final IWriter writer = IOFactory.newWriter();
		final IReader reader = IOFactory.newReader();

		int i = 0;
		
		try {

			for (i = 0; i < TEST_COUNT; ++i) {
				
				final String filename = "testfiles" + File.separator 
					+ "test" + i + "." + Main.GAME_EXTENSION;
				
				IGame g = reader.readGame(filename,
						IGame.MIN_WIDTH, IGame.MIN_HEIGHT, 20, 20);

				final String filenameout = "testfiles" + File.separator 
				+ "testout" + i + "." + Main.GAME_EXTENSION;

				writer.writeGame(g, filenameout);
			}
			
		} catch (VZabException e) {
			fail("Failed test number = " + i + "\n" +
					"Error: " + e.getMessage());
		} catch (RuntimeException e) {
			fail("Failed test number = " + i + "\n" +
					"Uncknown exception");
			e.printStackTrace();
		}
	}
	
	public void testBadGames() {
		final IReader reader = IOFactory.newReader();
		
		for (int i = 1; ; ++i) {
			try {
				String filename = String.format("./testfiles/.badgames/g%1$03d.vzg", i);
				
				File file = new File(filename);
				if (!file.exists()) {
					System.out.println("IOTests.testBadGames(): " + (i - 1) + " tests were found\n");
					break;
				}
				//System.out.println("IOTests.testBadGames(): filename = " + filename);
				
				reader.readGame(filename, IGame.MIN_WIDTH, IGame.MIN_HEIGHT, 20, 20);
			} catch (VZabException e) {
				
				System.out.println("IOTests.testBadGames(): Game number " + i + "\nmessage:" + e.getMessage() + "\n");
				
				continue;
				
			} catch (RuntimeException e) {
				
				e.printStackTrace();
				
				fail("Game number = " + i + "\nException message: " + e.getMessage());
			}
			
			fail("Bad game number " + i + " was successfully loaded");
		}
	}
	
	/*public void test01() {
		IReader reader = IOFactory.newReader();

		final String filename = ".testfiles" + File.separator 
			+ "xtest01." + Main.GAME_EXTENSION;
	
		IGame game = reader.readGame(filename,
			IGame.MIN_WIDTH, IGame.MIN_HEIGHT, 20, 20);
		
		assertEquals(1, game.getCurrentPlayer());
		
//		-------------------
		
		game.undoMove();
		assertEquals(0, game.getCurrentPlayer());

		game.redoMove();
		assertEquals(1, game.getCurrentPlayer());
		
//		-------------------
		
		game.undoMove();
		assertEquals(0, game.getCurrentPlayer());
		
		game.undoMove();
		assertEquals(2, game.getCurrentPlayer());
		
		game.undoMove();
		assertEquals(1, game.getCurrentPlayer());

		game.redoMove();
		assertEquals(2, game.getCurrentPlayer());
		
		game.redoMove();
		assertEquals(0, game.getCurrentPlayer());

		game.redoMove();
		assertEquals(1, game.getCurrentPlayer());

	}*/

	public void test02() {
		IPlayerIO io = IOFactory.newPlayerIO();
		
		IPlayer p1 = io.readPlayer("/players/blue-player/", "player1");
		
		assertEquals("player1", p1.getName());
		assertEquals("/players/blue-player/", p1.getPath());
		assertNotNull(p1.getImageAlive());
		assertNotNull(p1.getImageBase());
		assertNotNull(p1.getImageBaseKilled());
		assertNotNull(p1.getImageKilled());
		assertNotNull(p1.getImageSmall());
	}
	
	public void test03() {
		IPlayerIO io = IOFactory.newPlayerIO();
		
		IPlayer p1 = io.readPlayer("players/blue-player/", "player1");
		
		assertEquals("player1", p1.getName());
		assertEquals("/players/blue-player/", p1.getPath());
		assertNotNull(p1.getImageAlive());
		assertNotNull(p1.getImageBase());
		assertNotNull(p1.getImageBaseKilled());
		assertNotNull(p1.getImageKilled());
		assertNotNull(p1.getImageSmall());
	}

	
	public void test04() {
		IPlayerIO io = IOFactory.newPlayerIO();
		
		IPlayer p1 = io.readPlayer("/players/blue-player/", "player1");
		IPlayer p2 = io.readPlayer("/players/red-player/", "player2");
		
		assertFalse(p1.equals(p2));
		
		p1.setName("player1");
		
		assertFalse(p1.equals(p2));
		
		p2.setPath("/players/blue-player/");
		
		assertEquals(p1, p2);
	}
	
	public void test05() {
		IPlayerIO io = IOFactory.newPlayerIO();
		
		for (int i = 1; i < 6; ++i) {
			try {
				io.readPlayer("testfiles/badplayers/badplayer" + i + "/", "player1");
			} catch(VZabException e) {
				System.out.println("IOTests.test05(): message = " + e.getMessage() + "\n");
				continue;
			} catch(RuntimeException e) {
				fail("player number = " + i + "\nException message: " + e.getMessage());
			}
			
			fail("Bag player number " + i + " was successfully loaded");
		}
	}
}
