package org.amse.vbut.vzab.tests;

import junit.framework.*;
import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.model.impl.*;

public class ShellTests extends TestCase {

	public void test01() {
		IPlayer pl0 = ModelFactory.newPlayer("pl0", false, "/players/blue-player"); 
		
		IPlayerShell shell = ModelFactory.newPlayerShell(pl0, true);
		
		assertEquals("pl0", shell.getName());
		assertEquals("/players/blue-player", shell.getPath());
		assertEquals(pl0, shell.getParent());
		assertEquals(false, shell.isComputer());
		assertEquals(false, shell.isDeleted());
		assertEquals(true, shell.isNew());
		
		assertEquals(pl0, shell);
	}

	public void test02() {
		IPlayer pl0 = ModelFactory.newPlayer("pl0", false, "/players/blue-player"); 
		
		IPlayerShell shell = ModelFactory.newPlayerShell(pl0, true);
		
		shell.setName("ttt");
		assertEquals("ttt", shell.getName());

		assertEquals(pl0, shell);
		shell.applyChanges();
		assertEquals(pl0, shell);
		
		assertSame(pl0, shell.getParent());
	}

	public void test03() {
		IPlayer pl0 = ModelFactory.newPlayer("pl0", false, "/players/blue-player"); 
		
		IPlayerShell shell = ModelFactory.newPlayerShell(pl0, true);
		
		shell.setPath("/players/red-player");
		assertEquals("/players/red-player", shell.getPath());

		assertTrue(!pl0.equals(shell));
		shell.applyChanges();
		assertEquals(pl0, shell);
		
		assertSame(pl0, shell.getParent());
	}
	
	public void test04() {
		IPlayer pl0 = ModelFactory.newPlayer("pl0", false, "/players/blue-player"); 
		
		IPlayerShell shell = ModelFactory.newPlayerShell(pl0, true);
		
		shell.setComputer(true);
		assertEquals(true, shell.isComputer());

		assertEquals(pl0, shell);
		shell.applyChanges();
		assertEquals(pl0, shell);
		
		assertSame(pl0, shell.getParent());
	}

	public void test05() {
		IPlayer pl0 = ModelFactory.newPlayer("pl0", false, "/players/blue-player"); 
		
		IPlayerShell shell = ModelFactory.newPlayerShell(pl0, true);
		
		shell.setName("ttt");
		assertEquals("ttt", shell.getName());

		shell.setPath("/players/red-player");
		assertEquals("/players/red-player", shell.getPath());
		
		shell.setComputer(true);
		assertEquals(true, shell.isComputer());

		assertTrue(!pl0.equals(shell));
		shell.applyChanges();
		assertEquals(pl0, shell);
		
		assertSame(pl0, shell.getParent());
	}
	
	public void test06() {
		IPlayer pl0 = ModelFactory.newPlayer("pl0", false, "/players/blue-player"); 
		
		IPlayerShell shell = ModelFactory.newPlayerShell(pl0, true);
		
		shell.setName("ttt");
		assertEquals("ttt", shell.getName());

		shell.setPath("/players/red-player");
		assertEquals("/players/red-player", shell.getPath());
		
		shell.setComputer(true);
		assertEquals(true, shell.isComputer());

		assertTrue(!pl0.equals(shell));
		shell.applyChanges();
		assertEquals(pl0, shell);
		
		assertSame(pl0, shell.getParent());

		pl0.setName("xxx");
		assertNotSame(pl0, shell);
		
		shell.applyChanges();
		assertNotSame(pl0, shell);

		assertEquals(false, shell.isDeleted());
		shell.setDeleted(true);
		assertEquals(true, shell.isDeleted());

		assertEquals(pl0, shell);
		assertFalse( ModelFactory.newPlayerShell(pl0, true).equals(shell) );
		assertFalse( ModelFactory.newPlayerShell(pl0, false).equals(shell) );
		
		shell.applyChanges();
		assertEquals(pl0, shell);
	}
	
}
