package org.amse.vbut.vzab.view;

import java.awt.*;
import java.io.*;
import javax.swing.*;

import org.amse.vbut.vzab.*;
import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.io.*;
import org.amse.vbut.vzab.io.impl.*;

/**
 * Main class
 * 
 * @author But Vasiliy
 *
 */
@SuppressWarnings("serial")
public class Main {
	public static final String GAME_EXTENSION = "vzg";
	
	public static final String USER_DIR = System.getProperty("user.home");
	
	public static final String PLAYERS_FILENAME = USER_DIR + File.separator +  ".players.zab";
	public static final String SETTINGS_FILENAME = USER_DIR + File.separator +  ".settings.zab";
	
	public static final String IMAGES_PATH = "/images";
	public static final String PLAYERS_PATH = "/players";
	
	public static final int CELL_SIZE = IPlayer.IMAGE_SIZE + 2;
	
	private final String[] myPlayerNames = {"red-player", "blue-player", 
			"green-player", "yellow-player", "magenta-player", "purple-player"};
	
	
	static public int getMaxHeight() {
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		return d.height / CELL_SIZE - 3;
	}

	static public int getMaxWidth() {
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		return d.width / CELL_SIZE - 1;
	}
	
	public static void main(String[] args) {
		
		//JOptionPane.showMessageDialog(null, USER_DIR, "fff", JOptionPane.INFORMATION_MESSAGE);
		
		new Main();
	}
	
	public Main() {
		java.util.List<IPlayer> players;
		ISettings settings;

		IReader reader = IOFactory.newReader();
		try {
			players = reader.readPlayers(PLAYERS_FILENAME);
		} catch (VZabException e) {
			players = null;
		}
		
		try {
			settings = reader.readSettings(SETTINGS_FILENAME);
		} catch (VZabException e) {
			settings = reader.getDefaultSettings();
		}
		
		if (players == null) {
			
			players = new java.util.LinkedList<IPlayer>();

			IPlayerIO io = IOFactory.newPlayerIO();
			
			for (int i = 0; i < myPlayerNames.length; ++i) {
				try {
					players.add( io.readPlayer(PLAYERS_PATH + "/" + myPlayerNames[i], myPlayerNames[i]) );
				} catch (VZabException e) {
					JOptionPane.showMessageDialog(null, "Error occurred while loading players:\n" + e.getMessage(), 
							"Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			
			if (myPlayerNames.length < IGame.MIN_PLAYERS) {
				System.exit(-1);
			}
			
		}
		
		try {
			VPanel vpanel = new VPanel(IGame.DEFAULT_WIDTH, IGame.DEFAULT_HEIGHT);
			
			Viewer view = new Viewer(settings, vpanel, players);
			
			view.updateDimensions();
			view.updateView();
			
		} catch (VZabException e) {
			JOptionPane.showMessageDialog(null, "Error occurred while starting game:\n" + e.getMessage(), 
					"Error", JOptionPane.ERROR_MESSAGE);
			
			System.exit(-2);
		}
		
	}
	
}
