package org.amse.vbut.vzab.view;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.amse.vbut.vzab.*;
import org.amse.vbut.vzab.io.*;
import org.amse.vbut.vzab.io.impl.*;
import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.view.events.*;

/**
 * Viewer class
 * 
 * @author But Vasiliy
 *
 */
@SuppressWarnings("serial")
public class Viewer extends JFrame {

	private final VPanel myVPanel;
	private final java.util.List<IPlayer> myPlayers;
	
	private java.util.List<Action> myActions;
	
	private StatusBar myStatusBar;
	
	public VPanel getVPanel() {
		return myVPanel;
	}
	
	public java.util.List<IPlayer> getPlayers() {
		return myPlayers;
	}
	
	public Viewer(ISettings settings, VPanel panel, java.util.List<IPlayer> players) {
		super("VZab by But Vasiliy, AMSE 2007.");
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new VWindowListener());
		
		setLocation(settings.getMainWindowX(), settings.getMainWindowY());
		panel.setDieMessageShow(settings.isDieMessageShow());
		panel.setWinMessageShow(settings.isWinMessageShow());
		panel.setGrayMessageShow(settings.isGrayMessageShow());
	
		myPlayers = players;
		myVPanel = panel;
		myActions = new java.util.LinkedList<Action>();
		
		myStatusBar = new StatusBar(this);
		
		getContentPane().add(createMainToolBar(), BorderLayout.NORTH);
		getContentPane().add(panel);
		getContentPane().add(myStatusBar, BorderLayout.SOUTH);

		VMouseListener mouseListener = new VMouseListener(this);
		
		panel.addMouseListener(mouseListener);
		panel.addMouseMotionListener(mouseListener);

		pack();
		setResizable(false);
		setVisible(true);
		
		updateView();
	}
	
	private void addToToolBar(JToolBar tb, Action a) {
		myActions.add(a);
		tb.add(a);
	}
	
	private JToolBar createMainToolBar() {
		JToolBar toolbar = new JToolBar();
		addToToolBar(toolbar, new NewGameAction(this));
		addToToolBar(toolbar, new OpenGameAction(this));
		addToToolBar(toolbar, new SaveGameAction(this));
		toolbar.addSeparator();
		addToToolBar(toolbar, new UndoMoveAction(this));
		addToToolBar(toolbar, new RedoMoveAction(this));
		toolbar.addSeparator();
		addToToolBar(toolbar, new EditPlayersAction(this));
		toolbar.addSeparator();
		addToToolBar(toolbar, new AboutAction(this));
		
		toolbar.setFloatable(false);
		
		return toolbar;
	}
	
	public void updateView() {
		for (Action a: myActions) {
			a.setEnabled(a.isEnabled());
		}
		myStatusBar.updateStatusBar();
		repaint();
	}
	
	public void updateDimensions() {
		repaint();
		//setResizable(true);
		pack();
		//setResizable(false);
		repaint();
	}

	@SuppressWarnings("unused")
	private class VWindowListener extends WindowAdapter {
		
		public void windowClosing(WindowEvent we) {
			final IGame game = getVPanel().getGame();
			if (game != null && game.isModified()) {

				int result = JOptionPane.showConfirmDialog(Viewer.this, 
						"Current game is modified.\nDo you want to save it?", "Save game", 
						JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
				
				switch (result) {
				case JOptionPane.YES_OPTION: 
					if (!SaveGameAction.saveGame(Viewer.this)) {
						return;
					}
					break;
					
				case JOptionPane.CANCEL_OPTION:
					return;
				}

			}
			
			try {
				IWriter writer = IOFactory.newWriter();

				try {
					writer.writePlayers(getPlayers(), Main.PLAYERS_FILENAME);
				} catch (VZabException e) {
					JOptionPane.showMessageDialog(Viewer.this, "Error while saving players list: " + e.getMessage(),
							"Error", JOptionPane.ERROR_MESSAGE);
				}
	
				try {
					
					VPanel panel = Viewer.this.getVPanel();
					
					writer.writeSettings(Main.SETTINGS_FILENAME, Viewer.this.getX(), Viewer.this.getY(), 
							panel.isDieMessageShow(), panel.isGrayMessageShow(), panel.isWinMessageShow());
					
				} catch (VZabException e) {
					JOptionPane.showMessageDialog(Viewer.this, "Error while saving settings: " + e.getMessage(),
							"Error", JOptionPane.ERROR_MESSAGE);
				}
			} finally {
				System.exit(0);
			}

		}
		
	}
	
}
