package org.amse.vbut.vzab.view.dialogs;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

/*import java.io.*;

import org.amse.vbut.vzab.*;

import org.amse.vbut.vzab.io.*;
import org.amse.vbut.vzab.io.impl.IOFactory;*/

import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.model.impl.ModelFactory;

import org.amse.vbut.vzab.view.tools.*;

/**
 * EditPlayersListDialog class
 * 
 * @author But Vasiliy
 *
 */
@SuppressWarnings("serial")
public class EditPlayersListDialog extends JDialog {
	
	public final static String DEFAULT_PLAYER_NAME = "default-player";
	
	private java.util.List<IPlayer> myPlayers;
	private java.util.List<IPlayerShell> myShells;
	
	private DefaultListModel myListModel;
	private JList myPlayersList;
	
	private boolean myResult;
	
	private java.util.List<Action> myActions;
	
	public EditPlayersListDialog(Frame owner, java.util.List<IPlayer> players) {
		super(owner, "Edit players list", true);
		
		myPlayers = players;
		myShells = new java.util.LinkedList<IPlayerShell>();
		myActions = new java.util.LinkedList<Action>();
		myResult = false;
		
		for (IPlayer p: players) {
			myShells.add(ModelFactory.newPlayerShell(p, false));
		}
		
		getContentPane().add(createMainPanel(myShells));
	
		pack();
		
		setLocation( owner.getX() + (owner.getWidth() - getWidth()) / 2, 
				owner.getY() + (owner.getHeight() - getHeight()) / 2);
		
		setVisible(true);
		updateDialog();
	}
	
	private Action addToList(java.util.List<Action> list, Action a) {
		list.add(a);
		a.setEnabled(false);
		return a;
	}
	
	private JPanel createButtonsPanel() {
		JPanel panelButtons = BoxLayoutUtils.createVerticalPanel();
		
		/*Action a = addToList(myActions, new AddAction());
		JButton buttonAdd =    new JButton(a);
		a.setEnabled(true);
		JButton buttonDelete = new JButton(addToList(myActions, new DeleteAction()));*/
		JButton buttonEdit =   new JButton(addToList(myActions, new EditAction()));
		JButton buttonUp =     new JButton(addToList(myActions, new UpAction()));
		JButton buttonDown =   new JButton(addToList(myActions, new DownAction()));
		
		/*buttonAdd.setToolTipText("Add new player");
		buttonDelete.setToolTipText("Delete selected player");*/
		buttonEdit.setToolTipText("Edit selected player");
		buttonUp.setToolTipText("Move up selected player");
		buttonDown.setToolTipText("Move down selected player");
		
		/*panelButtons.add(buttonAdd);
		panelButtons.add(Box.createVerticalStrut(5));*/
		panelButtons.add(buttonEdit);
		/*panelButtons.add(Box.createVerticalStrut(5));
		panelButtons.add(buttonDelete);*/
		panelButtons.add(Box.createVerticalStrut(17));
		panelButtons.add(buttonUp);
		panelButtons.add(Box.createVerticalStrut(5));
		panelButtons.add(buttonDown);
		panelButtons.add(Box.createVerticalGlue());

		BoxLayoutUtils.setGroupAlignmentX(new JComponent[] {/*buttonAdd,*/
				buttonEdit, /*buttonDelete, */buttonUp, buttonDown}, 
				Component.RIGHT_ALIGNMENT);
		
		GUITools.createRecommendedMargin(new JButton[] {/*buttonAdd,*/
				buttonEdit, /*buttonDelete, */buttonUp, buttonDown});
		
		GUITools.makeSameSize(new JComponent[] {/*buttonAdd, */buttonEdit,
				/*buttonDelete, */buttonUp, buttonDown});

		panelButtons.setMaximumSize(panelButtons.getPreferredSize());
		panelButtons.setMinimumSize(panelButtons.getPreferredSize());
		
		panelButtons.setAlignmentY(Component.TOP_ALIGNMENT);

		return panelButtons;
	}
	
	private void updateDialog() {
		for (Action a: myActions) {
			a.setEnabled(a.isEnabled());
		}
	}
	
	private JScrollPane createPlayersPane(java.util.List<IPlayerShell> players) {
		myListModel = new DefaultListModel();
		for (IPlayer player: players) {
			myListModel.addElement(player);
		}
		
		myPlayersList = new JList(myListModel);
		myPlayersList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		myPlayersList.addListSelectionListener(new VListSelectionListener());
		myPlayersList.setToolTipText("List of players");
		myPlayersList.setCellRenderer(new PlayerListCellRenderer());
		
		JScrollPane panePlayers = new JScrollPane(myPlayersList);
		
		Dimension dim = panePlayers.getPreferredSize();
		panePlayers.setPreferredSize(new Dimension(dim.width + 68, dim.height));
		
		panePlayers.setAlignmentY(Component.TOP_ALIGNMENT);
		return panePlayers;
	}
	
	private JPanel createConfirmPanel() {
		JPanel panelFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		JPanel panelGrid = new JPanel(new GridLayout(1, 2, 5, 0));
		
		JButton buttonOk = new JButton(new OkAction());
		JButton buttonCancel = new JButton(new CancelAction());
		
		panelGrid.add(buttonOk);
		panelGrid.add(buttonCancel);
		panelFlow.add(panelGrid);
		
		panelFlow.setMinimumSize(panelFlow.getPreferredSize());
		
		Dimension dim = panelFlow.getMaximumSize();
		panelFlow.setMaximumSize(new Dimension(dim.width, panelFlow.getPreferredSize().height));

		BoxLayoutUtils.setGroupAlignmentY(new JComponent[] {buttonOk, 
				buttonCancel}, Component.CENTER_ALIGNMENT);
		GUITools.createRecommendedMargin(new JButton[] {buttonOk, buttonCancel});
		
		getRootPane().setDefaultButton(buttonOk);
		
		return panelFlow;
	}
	
	private JPanel createMainPanel(java.util.List<IPlayerShell> players) {
		JPanel panelMain = BoxLayoutUtils.createVerticalPanel();
		
		panelMain.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		
		JPanel panelTop = BoxLayoutUtils.createHorizontalPanel();
		
		panelTop.add(createPlayersPane(players));
		panelTop.add(Box.createHorizontalStrut(12));
		panelTop.add(createButtonsPanel());
		
		panelMain.add(panelTop);
		panelMain.add(Box.createVerticalStrut(17));
		panelMain.add(createConfirmPanel());
		
		return panelMain;
	}
	
	private class OkAction extends AbstractAction {
		public OkAction() {
			super("Ok");
		}
		
		public void actionPerformed(ActionEvent ae) {
			
			myPlayers.clear();
			
			for (IPlayer p: myShells) {
				final IPlayerShell s = (IPlayerShell)p;
				
				if (s.isDeleted()) {
					continue;
				}
				
				s.applyChanges();
				myPlayers.add(s.getParent());
			}
			
			myResult = true;
			
			dispose();
		}
		
	}
	
	private class CancelAction extends AbstractAction {
		public CancelAction() {
			super("Cancel");
		}
		
		public void actionPerformed(ActionEvent ae) {
			myResult = false;
			dispose();
		}
	}
	
	/*private class AddAction extends AbstractAction {
		public AddAction() {
			super("Add");
		}
		
		public void actionPerformed(ActionEvent ae) {
			IPlayerShell shell = ModelFactory.newPlayerShell("Player" + myListModel.getSize(),
					new VZabColor(0, 0, 0), false);
			
			EditPlayerDialog dlg = new EditPlayerDialog(
					EditPlayersListDialog.this, shell);
			
			if (dlg.getResult()) {
				myListModel.addElement(shell);
				myShells.add(shell);
				myPlayersList.setSelectedIndex(myListModel.size() - 1);
				EditPlayersListDialog.this.repaint();
			}
			
			JFileChooser  chooser = new JFileChooser(".");
			chooser.setMultiSelectionEnabled(true);
			chooser.setDialogTitle("Load players...");
			chooser.addChoosableFileFilter( new ExtensionFileFilter( Main.PLAYER_EXTENSION, 
					"VZab game file (*." + Main.PLAYER_EXTENSION + ")" ) );

			
			int result = chooser.showOpenDialog(EditPlayersListDialog.this);
			
			if (result == JFileChooser.APPROVE_OPTION) {
				
				try {
					IPlayerIO pr = IOFactory.newPlayerIO();
					
					java.io.File[] files = chooser.getSelectedFiles();
					
					for (java.io.File file: files) {
						
						String path;
						path = file.getAbsolutePath();
						IPlayer p;
						
						final int namestart = path.lastIndexOf(File.separatorChar);
						final int nameend =  path.lastIndexOf('.');
						String defname;
						if (namestart != -1 && nameend != -1) {
							defname = path.substring(namestart + 1, nameend);
						} else {
							defname = DEFAULT_PLAYER_NAME;
						}
						
						String name = JOptionPane.showInputDialog(EditPlayersListDialog.this, 
								"Enter players name for player:\n" + path, 
								defname);
						
						try {
							
							p = pr.readPlayer(path, name);
							
						} catch (VZabException e) {
							JOptionPane.showMessageDialog(EditPlayersListDialog.this, 
									"Error: Unable to load player:\n"
									+ "" + path + "\n"
									+ e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							continue;
						}
						
						IPlayerShell ps = ModelFactory.newPlayerShell(p, true);

						if (!myShells.contains(ps)) {
							myListModel.addElement(ps);
							myShells.add(ps);
						}
					}
					
					myPlayersList.setSelectedIndex(myListModel.size() - 1);
					EditPlayersListDialog.this.repaint();
					
				} catch (VZabException e) {
					JOptionPane.showMessageDialog(EditPlayersListDialog.this, 
							"Error: Unable to load players:\n" 
							+ e.getMessage(), "Error",
							JOptionPane.ERROR_MESSAGE);
				}
				
			}
			
			updateDialog();
		}
		
		public boolean isEnabled() {
			return true;
		}
	}*/
	
	private class EditAction extends AbstractAction {
		public EditAction() {
			super("Edit");
		}
		
		public void actionPerformed(ActionEvent ae) {
			final int selected = myPlayersList.getSelectedIndex();
			
			if (selected != -1) {
				final IPlayerShell shell = (IPlayerShell)myListModel.get(selected);
				
				EditPlayerDialog dlg = new EditPlayerDialog(
						EditPlayersListDialog.this, shell);
				
				if (dlg.getResult()) {
					EditPlayersListDialog.this.repaint();
				}
			}
			
			updateDialog();
		}
		
		public boolean isEnabled() {
			return myPlayersList.getSelectedIndex() != -1;
		}

	}
	
	/*private class DeleteAction extends AbstractAction {
		public DeleteAction() {
			super("Delete");
		}
		
		public void actionPerformed(ActionEvent ae) {
			int selected = myPlayersList.getSelectedIndex();
			if (selected != -1) {
				final IPlayerShell shell = (IPlayerShell)myListModel.get(selected);
				
				int option = JOptionPane.showConfirmDialog(
						EditPlayersListDialog.this, 
						"Do you really want to delete player \"" + shell.getName() + "\"",
						"VZab",
						JOptionPane.YES_NO_OPTION);
				
				if (option == JOptionPane.YES_OPTION) {
					myListModel.remove(selected);
					
					if (shell.isNew()) {
						myShells.remove(shell);
					} else {
						shell.setDeleted(true);
					}
					
					if (selected >= myListModel.size()) {
						selected = myListModel.size() - 1;
					}
					
					myPlayersList.setSelectedIndex(selected);
					
					EditPlayersListDialog.this.repaint();
				}
			}
			
			updateDialog();
		}
		
		public boolean isEnabled() {
			return myPlayersList.getSelectedIndex() != -1;
		}
	}*/
	
	private class UpAction extends AbstractAction {
		public UpAction() {
			super("Up");
		}
		
		public void actionPerformed(ActionEvent arg0) {
			final int selected = myPlayersList.getSelectedIndex();
			if (selected > 0) {
				final Object o1 = myListModel.get(selected);
				final Object o2 = myListModel.get(selected - 1);
				
				myListModel.set(selected, o2);
				myListModel.set(selected - 1, o1);
				
				final int i1 = myShells.indexOf(o1);
				final int i2 = myShells.indexOf(o2);
				
				myShells.set(i1, (IPlayerShell)o2);
				myShells.set(i2, (IPlayerShell)o1);
				
				myPlayersList.setSelectedIndex(selected - 1);
				EditPlayersListDialog.this.repaint();
			}
			
			updateDialog();
		}
		
		public boolean isEnabled() {
			return myPlayersList.getSelectedIndex() > 0;
		}
	}
	
	private class DownAction extends AbstractAction {
		public DownAction() {
			super("Down");
		}
		
		public void actionPerformed(ActionEvent arg0) {
			final int selected = myPlayersList.getSelectedIndex();
			if (selected != - 1 && selected < (myListModel.getSize() - 1)) {
				final Object o1 = myListModel.get(selected);
				final Object o2 = myListModel.get(selected + 1);
				
				myListModel.set(selected, o2);
				myListModel.set(selected + 1, o1);
				
				final int i1 = myShells.indexOf(o1);
				final int i2 = myShells.indexOf(o2);
				
				myShells.set(i1, (IPlayerShell)o2);
				myShells.set(i2, (IPlayerShell)o1);

				myPlayersList.setSelectedIndex(selected + 1);
				EditPlayersListDialog.this.repaint();
			}
			
			updateDialog();
		}
		
		public boolean isEnabled() {
			final int selected = myPlayersList.getSelectedIndex();
			return selected != - 1 && selected < (myListModel.getSize() - 1);
		}
	}
	
	private class VListSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent arg0) {
			updateDialog();
		}
	}
	
	public boolean getResult() {
		return myResult;
	}
}
