package org.amse.vbut.vzab.view.events;

import java.io.File;
import java.awt.event.ActionEvent;

import javax.swing.*;

import org.amse.vbut.vzab.*;
import org.amse.vbut.vzab.io.*;
import org.amse.vbut.vzab.io.impl.*;
import org.amse.vbut.vzab.view.*;
import org.amse.vbut.vzab.model.*;

@SuppressWarnings("serial")
public class SaveGameAction extends AbstractAction {

	private Viewer myView;
	
	public SaveGameAction(Viewer view) {
		super("save game", IOTools.loadIcon(Main.IMAGES_PATH + "/save.png"));
		putValue(AbstractAction.SHORT_DESCRIPTION, "Save current game");
		myView = view;
	}
	
	public static boolean saveGame(Viewer view) {
		JFileChooser  chooser = new JFileChooser(".");
		chooser.setMultiSelectionEnabled(false);
		chooser.setDialogTitle("Save current game...");
		chooser.addChoosableFileFilter( new ExtensionFileFilter( Main.GAME_EXTENSION, 
				"VZab game file (*." + Main.GAME_EXTENSION + ")" ) );
		
		int result = chooser.showSaveDialog(view);
		
		if (result == JFileChooser.APPROVE_OPTION) {
			
			File file = chooser.getSelectedFile();
			String name = file.getName();
			String filename = file.getAbsolutePath();
			
			if (name.indexOf('.', 1) == -1) {
				filename += '.' + Main.GAME_EXTENSION;
			}
			
			if (new File(filename).exists()) {
				if (JOptionPane.showConfirmDialog(view, 
						"file \"" + filename + "\" already exists.\nReplace it?", 
						"confirm replace", 
						JOptionPane.YES_NO_OPTION, 
						JOptionPane.WARNING_MESSAGE) != JOptionPane.YES_OPTION) {
					return false;
				}
			}
			
			try {
				IWriter writer = IOFactory.newWriter();
				writer.writeGame(view.getVPanel().getGame(), filename);
			} catch (VZabException e) {
				JOptionPane.showMessageDialog(view, 
						"Unable to save game to file:\n\"" 
						+ chooser.getSelectedFile().getAbsolutePath() 
						+ "\".\nError: " + e.getMessage(),
						"Error",
						JOptionPane.ERROR_MESSAGE);
				return false;
			}
		
			view.getVPanel().setModified(false);
			return true;
		}
		
		return false;
	}
	
	public void actionPerformed(ActionEvent arg0) {
		saveGame(myView);
		myView.updateView();
	}
	
	public boolean isEnabled() {
		final IGame game = myView.getVPanel().getGame();
		return game != null && game.isModified();
	}

}
